#include "Application.hpp"
#include "GameState.hpp"
#include "PauseState.hpp"
#include "TitleState.hpp"
#include "GameOverState.hpp"
#include <cassert>

float lastScore = 0;
float bestScore = 0;

Settings::ARMap Settings::AspectRatios = {
	{"4x3", 0},
	{"16x9", 1},
	{"8x5", 2},
	{"16x10", 2}
};

Settings::Settings(std::string AspectRatio,
	sf::Vector2u WindowSize,
	bool Fullscreen) :
	aspectRatio(AspectRatios[AspectRatio]),
	windowSize(WindowSize),
	fullscreen(Fullscreen) {

}




Context::Context(sf::RenderWindow& window,
	TextureStorage& textureStorage,
	FontStorage& fontStorage,
	AnimationStorage& animationStorage,
	SoundPlayer& soundPlayer) :
	renderWindow(&window),
	textures(&textureStorage),
	fonts(&fontStorage),
	animations(&animationStorage),
	soundPlayer(&soundPlayer)
{}

Context::Context() {}




// Initalize static members
Settings Application::settings("16x9",
	sf::Vector2u(1280, 600),
	false);


void Application::load() {
	try {
		textures.Load(Texture::Clouds, "res/clouds.png");
		textures.Get(Texture::Clouds).setRepeated(true);

		textures.Load(Texture::Snow, "res/snoflake.png");
		textures.Load(Texture::BloodParticle, "res/bloodParticle.png");
		textures.Load(Texture::WaterParticle, "res/waterParticle.png");
			
		textures.Load(Texture::ReturnArrow, "res/returnarrow.png");

		textures.Load(Texture::Coin, "res/coin.png");

		textures.Load(Texture::TopCliff, "res/clifftop.png");
		textures.Load(Texture::BottomCliff, "res/cliffbottom.png");

		textures.Load(Texture::WaterSplash, "res/watersplash.png");

		textures.Load(Texture::TitleSplash, "res/titleSplash.png");
		textures.Load(Texture::BZepIco, "res/BZep.png");
		textures.Get(Texture::BZepIco).setSmooth(true);

		fonts.Load(Font::Arial, "res/arial.ttf");
		fonts.Load(Font::Emulator, "res/Emulator.ttf");

		AnimationStruct eagle(sf::Vector2u(2, 1), 300, false, "res/eagle.png");
		animations.insert(std::make_pair(Animation::Eagle, eagle));

		AnimationStruct fish(sf::Vector2u(2, 1), 300, true, "res/fish.png");
		animations.insert(std::make_pair(Animation::Fish, fish));

	} catch(std::runtime_error e) {

		throw;

	}

}


Application::Application() :
window(sf::VideoMode(settings.windowSize.x, settings.windowSize.y), "Ludum Dare 31", sf::Style::Close),
textures(),
fonts(),/*
animations(),
tilesets(),*/
stateStack(Context(window, textures, fonts, animations, soundPlayer)) {

	window.setKeyRepeatEnabled(false);

	try {
		load();

		registerStates();
		stateStack.PushState(States::TITLESTATE);
	} catch(std::runtime_error e) {
		assert(false); // resource loading error
	}
}


int Application::run() {

	sf::Clock dClock;
	const sf::Time frameCap = sf::seconds(1.f / 15.f);

	while(window.isOpen()) {
		sf::Time dt = std::min(dClock.restart(), frameCap);

		/* Implement some sort of fixed-time step here */
		handleEvents();
		update(dt);
		render();

		if(stateStack.IsEmpty())
			window.close();
	}


	return 0;
}

void Application::handleEvents() {

	sf::Event event;
	while(window.pollEvent(event)) {
		stateStack.HandleEvent(event);

		if(event.type == sf::Event::Closed)
			window.close();
	}

}


void Application::update(sf::Time& dt) {
	stateStack.Update(dt);
}


void Application::render() {

	window.clear(sf::Color(31, 38, 119));

	stateStack.Draw(window);

	window.setView(window.getDefaultView());

	window.display();

}


void Application::registerStates() {

	stateStack.RegisterState<TitleState>(States::TITLESTATE);
	stateStack.RegisterState<GameState>(States::GAMESTATE);
	stateStack.RegisterState<PauseState>(States::PAUSESTATE);
	stateStack.RegisterState<GameOverState>(States::GAMEOVERSTATE);
}